@@ -1,8 +1,8 @@
 /*
  * registry.c
- * $Id: registry.c 27967 2007-08-16 20:14:31Z sfiera@macports.org $
  *
  * Copyright (c) 2007 Chris Pickel <sfiera@macports.org>
+ * Copyright (c) 2012, 2014 The MacPorts Project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,17 +32,20 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <tcl.h>
 
 #include <cregistry/registry.h>
+#include <cregistry/portgroup.h>
 #include <cregistry/entry.h>
+#include <cregistry/file.h>
 
-#include "registry.h"
-#include "graph.h"
-#include "item.h"
 #include "entry.h"
 #include "entryobj.h"
+#include "file.h"
+#include "portgroup.h"
+#include "registry.h"
 #include "util.h"
 
 int registry_failed(Tcl_Interp* interp, reg_error* errPtr) {
@@ -53,6 +56,8 @@ int registry_failed(Tcl_Interp* interp, reg_error* errPtr) {
     return TCL_ERROR;
 }
 
+/* we don't need delete_file_list and restore_file_list unless we allow deletion
+   of files via the file interface */
 static void delete_entry_list(ClientData list, Tcl_Interp* interp UNUSED) {
     entry_list* curr = *(entry_list**)list;
     while (curr) {
@@ -79,14 +84,43 @@ static void restore_entry_list(ClientData list, Tcl_Interp* interp) {
 int registry_tcl_detach(Tcl_Interp* interp, reg_registry* reg,
         reg_error* errPtr) {
     reg_entry** entries;
-    int entry_count = reg_all_open_entries(reg, &entries);
+    reg_file** files;
+    reg_portgroup** portgroups;
+    int count;
     int i;
-    for (i=0; i<entry_count; i++) {
+
+    count = reg_all_open_entries(reg, &entries);
+    if (count == -1) {
+        return 0;
+    }
+    for (i = 0; i < count; i++) {
         if (entries[i]->proc) {
             Tcl_DeleteCommand(interp, entries[i]->proc);
         }
     }
     free(entries);
+
+    count = reg_all_open_files(reg, &files);
+    if (count == -1) {
+        return 0;
+    }
+    for (i = 0; i < count; i++) {
+        if (files[i]->proc) {
+            Tcl_DeleteCommand(interp, files[i]->proc);
+        }
+    }
+    free(files);
+
+    count = reg_all_open_portgroups(reg, &portgroups);
+    if (count == -1) {
+        return 0;
+    }
+    for (i = 0; i < count; i++) {
+        if (portgroups[i]->proc) {
+            Tcl_DeleteCommand(interp, portgroups[i]->proc);
+        }
+    }
+    free(portgroups);
     if (!reg_detach(reg, errPtr)) {
         return registry_failed(interp, errPtr);
     }
@@ -103,20 +137,29 @@ int registry_tcl_detach(Tcl_Interp* interp, reg_registry* reg,
  *
  * Then it will leak memory :(
  */
-static void delete_reg(ClientData reg, Tcl_Interp* interp UNUSED) {
+static void delete_reg(ClientData reg, Tcl_Interp* interp) {
     reg_error error;
     if (((reg_registry*)reg)->status & reg_attached) {
+        if (Tcl_GetAssocData(interp, "registry::needs_vacuum", NULL) != NULL) {
+            reg_vacuum(Tcl_GetAssocData(interp, "registry::db_path", NULL));
+            Tcl_DeleteAssocData(interp, "registry::needs_vacuum");
+        }
         if (!registry_tcl_detach(interp, (reg_registry*)reg, &error)) {
-            fprintf(stderr, error.description);
+            fprintf(stderr, "%s", error.description);
             reg_error_destruct(&error);
         }
     }
     if (!reg_close((reg_registry*)reg, &error)) {
-        fprintf(stderr, error.description);
+        fprintf(stderr, "%s", error.description);
         reg_error_destruct(&error);
     }
 }
 
+/* simple destructor for malloc()ed assoc data */
+static void free_assoc_data(ClientData ptr, Tcl_Interp* interp UNUSED) {
+    free(ptr);
+}
+
 /**
  * Returns the sqlite3 DB associated with interp.
  *
@@ -157,7 +200,7 @@ reg_registry* registry_for(Tcl_Interp* interp, int status) {
 }
 
 static int registry_open(ClientData clientData UNUSED, Tcl_Interp* interp,
-        int objc, Tcl_Obj* CONST objv[]) {
+        int objc, Tcl_Obj* const objv[]) {
     if (objc != 2) {
         Tcl_WrongNumArgs(interp, 1, objv, "db-file");
         return TCL_ERROR;
@@ -165,9 +208,14 @@ static int registry_open(ClientData clientData UNUSED, Tcl_Interp* interp,
         char* path = Tcl_GetString(objv[1]);
         reg_registry* reg = registry_for(interp, 0);
         reg_error error;
+        if (Tcl_GetAssocData(interp, "registry::db_path", NULL) == NULL) {
+            char *pathCopy = strdup(path);
+            Tcl_SetAssocData(interp, "registry::db_path", free_assoc_data, pathCopy);
+        }
         if (reg == NULL) {
             return TCL_ERROR;
         } else if (reg_attach(reg, path, &error)) {
+            reg_configure(reg);
             return TCL_OK;
         } else {
             return registry_failed(interp, &error);
@@ -176,7 +224,7 @@ static int registry_open(ClientData clientData UNUSED, Tcl_Interp* interp,
 }
 
 static int registry_close(ClientData clientData UNUSED, Tcl_Interp* interp,
-        int objc, Tcl_Obj* CONST objv[]) {
+        int objc, Tcl_Obj* const objv[]) {
     if (objc != 1) {
         Tcl_WrongNumArgs(interp, 1, objv, NULL);
         return TCL_ERROR;
@@ -186,6 +234,14 @@ static int registry_close(ClientData clientData UNUSED, Tcl_Interp* interp,
             return TCL_ERROR;
         } else {
             reg_error error;
+            if (Tcl_GetAssocData(interp, "registry::needs_vacuum", NULL) != NULL) {
+                reg_vacuum(Tcl_GetAssocData(interp, "registry::db_path", NULL));
+                Tcl_DeleteAssocData(interp, "registry::needs_vacuum");
+            }
+            /* Not really anything we can do if this fails. */
+            if (reg_checkpoint(reg, &error) == 0) {
+                fprintf(stderr, "%s\n", error.description);
+            }
             if (registry_tcl_detach(interp, reg, &error)) {
                 return TCL_OK;
             }
@@ -196,7 +252,7 @@ static int registry_close(ClientData clientData UNUSED, Tcl_Interp* interp,
 }
 
 static int registry_read(ClientData clientData UNUSED, Tcl_Interp* interp,
-        int objc, Tcl_Obj* CONST objv[]) {
+        int objc, Tcl_Obj* const objv[]) {
     if (objc != 2) {
         Tcl_WrongNumArgs(interp, 1, objv, "command");
         return TCL_ERROR;
@@ -232,7 +288,7 @@ static int registry_read(ClientData clientData UNUSED, Tcl_Interp* interp,
 }
 
 static int registry_write(ClientData clientData UNUSED, Tcl_Interp* interp,
-        int objc, Tcl_Obj* CONST objv[]) {
+        int objc, Tcl_Obj* const objv[]) {
     if (objc != 2) {
         Tcl_WrongNumArgs(interp, 1, objv, "command");
         return TCL_ERROR;
@@ -281,6 +337,57 @@ static int registry_write(ClientData clientData UNUSED, Tcl_Interp* interp,
     }
 }
 
+/*
+ * registry::metadata cmd ?arg ...?
+ *
+ * Commands manipulating metadata in the registry. This can be called `registry::metadata`
+ */
+int metadata_cmd(ClientData clientData UNUSED, Tcl_Interp* interp, int objc,
+        Tcl_Obj* const objv[]) {
+    if (objc < 3) {
+        Tcl_WrongNumArgs(interp, 1, objv, "cmd key ?value?");
+        return TCL_ERROR;
+    }
+    reg_registry* reg = registry_for(interp, reg_attached);
+    if (reg == NULL) {
+        return TCL_ERROR;
+    }
+    const char *cmdstring = Tcl_GetString(objv[1]);
+    reg_error error;
+    if (strcmp(cmdstring, "get") == 0) {
+        char *data;
+        if (reg_get_metadata(reg, Tcl_GetString(objv[2]), &data, &error)) {
+            Tcl_Obj* result = Tcl_NewStringObj(data, -1);
+            Tcl_SetObjResult(interp, result);
+            free(data);
+            return TCL_OK;
+        } else if (error.code == REG_NOT_FOUND) {
+            Tcl_Obj* result = Tcl_NewIntObj(-1);
+            Tcl_SetObjResult(interp, result);
+            return TCL_OK;
+        } else {
+            return registry_failed(interp, &error);
+        }
+    } else if (strcmp(cmdstring, "set") == 0) {
+        if (objc < 4) {
+            Tcl_WrongNumArgs(interp, 1, objv, "set key value");
+            return TCL_ERROR;
+        }
+        if (reg_set_metadata(reg, Tcl_GetString(objv[2]), Tcl_GetString(objv[3]), &error)) {
+            return TCL_OK;
+        } else {
+            return registry_failed(interp, &error);
+        }
+    } else if (strcmp(cmdstring, "del") == 0) {
+        if (reg_del_metadata(reg, Tcl_GetString(objv[2]), &error)) {
+            return TCL_OK;
+        } else {
+            return registry_failed(interp, &error);
+        }
+    }
+    return TCL_ERROR;
+}
+
 /**
  * Initializer for the registry lib.
  *
@@ -288,16 +395,17 @@ static int registry_write(ClientData clientData UNUSED, Tcl_Interp* interp,
  * It creates the global commands made available in the registry namespace.
  */
 int Registry_Init(Tcl_Interp* interp) {
-    if (Tcl_InitStubs(interp, "8.3", 0) == NULL) {
+    if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
         return TCL_ERROR;
     }
     Tcl_CreateObjCommand(interp, "registry::open", registry_open, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::close", registry_close, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::read", registry_read, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::write", registry_write, NULL, NULL);
-    /* Tcl_CreateObjCommand(interp, "registry::graph", GraphCmd, NULL, NULL); */
-    /* Tcl_CreateObjCommand(interp, "registry::item", item_cmd, NULL, NULL); */
     Tcl_CreateObjCommand(interp, "registry::entry", entry_cmd, NULL, NULL);
+    Tcl_CreateObjCommand(interp, "registry::file", file_cmd, NULL, NULL);
+    Tcl_CreateObjCommand(interp, "registry::portgroup", portgroup_cmd, NULL, NULL);
+    Tcl_CreateObjCommand(interp, "registry::metadata", metadata_cmd, NULL, NULL);
     if (Tcl_PkgProvide(interp, "registry2", "2.0") != TCL_OK) {
         return TCL_ERROR;
     }
